/* Shortest and probably fastest Hanoi Tower algorithm */

/* - Found on the web, and then improved by Glenn C. Rhoads,
				 rhoads@paul.rutgers.edu
   - M. Kolar (2000) changed the input, eliminated the remainder
   operator (%), and added the (optional) while loop generating
   the number of the disk to be moved. This loop slows down the
   algorithm considerably. But it is a minimal addition needed
   for comparison with other TH algorithms, some of which generate
   even more information - occupation of all three pegs.  
*/

/* Start post is post 0. */
/* If # disks even, then final post is post 1, else is post 2 */

#include <stdio.h>
#include <stdlib.h>

main(int argc, char *argv[])
/*========================*/
{
 int n, x, fr, to;
 unsigned int i, j;

 n = atoi(argv[1]);

 for (x=1; x < (1 << n); x++) {

   i=(x|x-1)+1; to=(i+i/3)&3;
   i=x&x-1; fr=(i+i/3)&3;
   /* fr=(x&x-1)%3; to=((x|x-1)+1)%3; */

   for(i=x, j=1; ; i>>=1, j++) { if(i&1) break; }

   /* printf( "move disk %i from %i to %i\n", j, fr, to ); */
 }

}

/*
Above, the line
   for(i=x, j=1; ; i>>=1, j++) { if(i&1) break; }
calculates the number, j, of the disk being moved in move  x .
j is by 1 larger than the number of the rightmost 0 bits of  x .
This line can be replaced by
   i=x; j=1; while(!(i&1)) { i>>=1; j++; }
which may be, by a really tiny bit, faster for some compilers
in spite of the fact that it contains one more operation (!).

If neither of these two lines is included, and the previous code
fragment for calculation of the from and to peg numbers were just
the original (see below)
   (x&x-1)%3; ((x|x-1)+1)%3;
(to find the CPU time of the "bare" algorithm without printing),
this lines could be "optimized away" for some compilers, resulting
in zero CPU time. That's why the assignment to the  fr  and  to
variables is neccessary.

The remainder operator (%) is very slow for some compilers. That's
why i%3 is replaced above by the identical (i+i/3)&3. This results
in four times faster code for DEC C compiler, but only nebligible
speed up for gcc. Actually, this was true only before the for
loop that follows was added!
*/



/* The original code I got from G.C.R:
   ----------------------------------

void main(void)
   {
   int n, x;

   printf( "How many disks? " );
   scanf( "%d", &n );
   puts( "\n\n" );

   for (x=1; x < (1 << n); x++)
      printf( "move from pole %i to pole %i.\n", (x&x-1)%3, ((x|x-1)+1)%3 );
   }
*/